/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/core.h"
#include "gctl/algorithm.h"
#include "../lib/optimization.h"

// get random floating points
double random_double(double l, double t)
{
	return (t-l)*rand()*1.0/RAND_MAX + l;
}

int main(int argc, char const *argv[])
{
	srand(time(0));

	gctl::matrix<double> A(5, 5);
	for (int i = 0; i < 5; i++)
	{
		for (int j = i; j < 5; j++)
		{
			if (i == j) A[i][j] = random_double(1.0, 3.0);
			else A[i][j] = random_double(0.1, 1.0);
		}
	}

	for (int i = 0; i < 5; i++)
	{
		for (int j = i; j < 5; j++)
		{
			A[j][i] = A[i][j];
		}
	}

	std::cout<<"A(5, 5) = " <<std::endl;
	for(int i=0;i<5;i++){
		for(int j=0;j<5;j++){
			std::cout<<A[i][j]<<' ';
		}
		std::cout<<std::endl;
	}
	std::cout<<std::endl;

    gctl::array<double> m(5, 0.5), x(5, 0.0);
    gctl::array<double> B(5);
    for (int i = 0; i < 5; i++)
	{
        B[i] = 0.0;
		for (int j = 0; j < 5; j++)
		{
			B[i] += A[i][j] * m[j];
		}
	}

	gctl::cholesky gck(A);
	gck.decompose();
    gck.solve(B, x);

    for (size_t i = 0; i < 5; i++)
    {
        std::cout << m[i] << " " << x[i] << std::endl;
    }

	return 0;
}