/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_LU_H
#define _GCTL_LU_H

#include "gctl/core.h"

namespace gctl
{
    /**
     * @brief      Lower/upper decomposition of matrix into a lower triangular matrix and a upper triangular matrix.
     *
     * @tparam     T     template type
     */
    class lu
    {
    public:
        lu(matrix<double> &sourceMatrix); // Matrix is decomposed in-place
        virtual ~lu(){}
        void decompose(); ///< Decomposition into triangular matrices. Return false if failed
        void solve(const array<double>& b, array<double> &x); ///< solve for x in form Ax = b.  A is the original input matrix.

    protected:
        lu(const lu&) = delete;    
        void operator=(const lu&) = delete;
        matrix<double> &decomposedMatrix; // Output matrix after decomposition
        array<int> rowPermutation; // Permutation of rows during pivoting
    };
}

#endif // _GCTL_LU_H